package com.nordlicht.views.empty;

import com.nordlicht.views.main.MainView;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.board.Board;
import com.vaadin.flow.component.charts.Chart;
import com.vaadin.flow.component.charts.model.Configuration;
import com.vaadin.flow.component.charts.model.DataGrouping;
import com.vaadin.flow.component.charts.model.DataSeries;
import com.vaadin.flow.component.charts.model.DataSeriesItem;
import com.vaadin.flow.component.charts.model.OhlcItem;
import com.vaadin.flow.component.charts.model.PlotOptionsOhlc;
import com.vaadin.flow.component.charts.model.TimeUnit;
import com.vaadin.flow.component.charts.model.TimeUnitMultiples;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.gridpro.GridPro;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Span;
import com.vaadin.flow.data.renderer.ComponentRenderer;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.router.AfterNavigationEvent;
import com.vaadin.flow.router.AfterNavigationObserver;
import com.vaadin.flow.router.BeforeLeaveEvent;
import com.vaadin.flow.router.BeforeLeaveObserver;
import com.vaadin.flow.router.PageTitle;
import com.vaadin.flow.router.Route;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

@Route(value = "empty", layout = MainView.class)
@PageTitle("Empty")
@CssImport("styles/views/empty/empty-view.css")
public class EmptyView extends Div implements AfterNavigationObserver, BeforeLeaveObserver {

    private final AtomicInteger sleepTimeoutMax = new AtomicInteger(600);
    private final AtomicInteger numberOfItemsInGrid = new AtomicInteger(18);
    private final AtomicInteger maxItemsInChart = new AtomicInteger(50);
    private final AtomicInteger numberofUpdatedItemsAtOnce = new AtomicInteger(3);
    private final AtomicLong timestampBase = new AtomicLong();
    private final Random random = new Random();
    private final LinkedList<Map<String, Span>> rowsGrid1 = new LinkedList<>();
    private final LinkedList<Map<String, Span>> rowsGrid2 = new LinkedList<>();
    private final Chart priceChart1 = new Chart();
    private final Chart priceChart2 = new Chart();
    private final DataSeries dataSeries1 = new DataSeries();
    private final DataSeries dataSeries2 = new DataSeries();
    private final GridPro<MarketData> grid1 = new GridPro<>();
    private final GridPro<MarketData> grid2 = new GridPro<>();

    private final AtomicBoolean shouldStop = new AtomicBoolean();
    private final AtomicInteger columnCount = new AtomicInteger();

    private static final String COLUMN_TIMESTAMP = "Timestamp";
    private static final String COLUMN_BROKER = "Broker";
    private static final String COLUMN_EXCHANGE = "Exchange";
    private static final String COLUMN_SYMBOL = "Symbol";
    private static final String COLUMN_CURRENCY = "Currency";
    private static final String COLUMN_PRICE_TYPE = "Price type";
    private static final String COLUMN_EVENT_TYPE = "Event type";
    private static final String COLUMN_PRICE = "Price";
    private static final String COLUMN_VOLUME = "Volume";

    public EmptyView() {
        setSizeFull();
        setId("empty-view");

        grid1.addColumn(createCell(rowsGrid1, COLUMN_TIMESTAMP, marketData -> formatDate(marketData.getTimestamp()))).setHeader(COLUMN_TIMESTAMP);
        grid1.addColumn(createCell(rowsGrid1, COLUMN_BROKER, marketData -> marketData.getBroker().name())).setHeader(COLUMN_BROKER);
        grid1.addColumn(createCell(rowsGrid1, COLUMN_EXCHANGE, marketData -> marketData.getExchange().name())).setHeader(COLUMN_EXCHANGE);
        grid1.addColumn(createCell(rowsGrid1, COLUMN_SYMBOL, MarketData::getSymbol)).setHeader(COLUMN_SYMBOL);
        grid1.addColumn(createCell(rowsGrid1, COLUMN_CURRENCY, MarketData::getCurrency)).setHeader(COLUMN_CURRENCY);
        grid1.addColumn(createCell(rowsGrid1, COLUMN_PRICE_TYPE, marketData -> marketData.getPriceType().name())).setHeader(COLUMN_PRICE_TYPE);
        grid1.addColumn(createCell(rowsGrid1, COLUMN_EVENT_TYPE, marketData -> marketData.getEventType().name())).setHeader(COLUMN_EVENT_TYPE);
        grid1.addColumn(createCell(rowsGrid1, COLUMN_PRICE, marketData -> marketData.getPrice().toString())).setHeader(COLUMN_PRICE);
        grid1.addColumn(createCell(rowsGrid1, COLUMN_VOLUME, marketData -> marketData.getVolume().toString())).setHeader(COLUMN_VOLUME);
        grid1.addThemeVariants(GridVariant.LUMO_NO_BORDER, GridVariant.LUMO_COMPACT, GridVariant.LUMO_ROW_STRIPES, GridVariant.LUMO_COLUMN_BORDERS, GridVariant.LUMO_WRAP_CELL_CONTENT);
        grid1.setItems(getRandomMarketData(numberOfItemsInGrid.get()));
        grid1.setHeight("50vh");
        grid1.setWidth("50vw");

        grid2.addColumn(createCell(rowsGrid2, COLUMN_TIMESTAMP, marketData -> formatDate(marketData.getTimestamp()))).setHeader(COLUMN_TIMESTAMP);
        grid2.addColumn(createCell(rowsGrid2, COLUMN_BROKER, marketData -> marketData.getBroker().name())).setHeader(COLUMN_BROKER);
        grid2.addColumn(createCell(rowsGrid2, COLUMN_EXCHANGE, marketData -> marketData.getExchange().name())).setHeader(COLUMN_EXCHANGE);
        grid2.addColumn(createCell(rowsGrid2, COLUMN_SYMBOL, MarketData::getSymbol)).setHeader(COLUMN_SYMBOL);
        grid2.addColumn(createCell(rowsGrid2, COLUMN_CURRENCY, MarketData::getCurrency)).setHeader(COLUMN_CURRENCY);
        grid2.addColumn(createCell(rowsGrid2, COLUMN_PRICE_TYPE, marketData -> marketData.getPriceType().name())).setHeader(COLUMN_PRICE_TYPE);
        grid2.addColumn(createCell(rowsGrid2, COLUMN_EVENT_TYPE, marketData -> marketData.getEventType().name())).setHeader(COLUMN_EVENT_TYPE);
        grid2.addColumn(createCell(rowsGrid2, COLUMN_PRICE, marketData -> marketData.getPrice().toString())).setHeader(COLUMN_PRICE);
        grid2.addColumn(createCell(rowsGrid2, COLUMN_VOLUME, marketData -> marketData.getVolume().toString())).setHeader(COLUMN_VOLUME);
        grid2.addThemeVariants(GridVariant.LUMO_NO_BORDER, GridVariant.LUMO_COMPACT, GridVariant.LUMO_ROW_STRIPES, GridVariant.LUMO_NO_ROW_BORDERS);
        grid2.setItems(getRandomMarketData(numberOfItemsInGrid.get()));
        grid2.setHeight("50vh");
        grid2.setWidth("50vw");

        Configuration configuration = priceChart1.getConfiguration();
        configuration.getTooltip().setEnabled(true);
        MarketData randomMarketData = getRandomMarketData();
        configuration.getTitle().setText("Last Price " + randomMarketData.getSymbol() + " / " + randomMarketData.getCurrency());
        configuration.addSeries(dataSeries1);
        priceChart1.setHeight("50vh");
        priceChart1.setWidth("50vw");

        configuration = priceChart2.getConfiguration();
        configuration.getTooltip().setEnabled(true);
        randomMarketData = getRandomMarketData();
        configuration.getTitle().setText("Last Price " + randomMarketData.getSymbol() + " / " + randomMarketData.getCurrency());
        configuration.addSeries(dataSeries2);

        PlotOptionsOhlc plotOptionsOhlc = new PlotOptionsOhlc();
        DataGrouping grouping = new DataGrouping();
        grouping.addUnit(new TimeUnitMultiples(TimeUnit.MINUTE, 1));
        grouping.addUnit(new TimeUnitMultiples(TimeUnit.SECOND, 1, 2, 3, 4, 6));
        plotOptionsOhlc.setDataGrouping(grouping);
        dataSeries2.setPlotOptions(plotOptionsOhlc);
        priceChart2.setHeight("50vh");
        priceChart2.setWidth("50vw");



        Board board = new Board();
        board.setMaxWidth("none");
        board.setMaxHeight("none");
        board.setMinHeight("1200px");
        board.addRow(grid1, priceChart1);
        board.addRow(priceChart2, grid2);
        board.setSizeFull();
        add(board);
        columnCount.set(grid1.getColumns().size());
    }

    private ComponentRenderer<Span, MarketData> createCell(LinkedList<Map<String, Span>> rows, String column, ValueProvider<MarketData, String> provider) {
        return new ComponentRenderer<>(item -> {
            Span span = new Span(provider.apply(item));
            if (rows.isEmpty() || rows.getLast().size() == columnCount.get()) {
                rows.add(new HashMap<>());
            }
            rows.getLast().put(column, span);
            return span;
        });
    }

    private void addDataSeriesPoints(DataSeries dataSeries, List<MarketData> md) {
        Collections.sort(md);
        OhlcItem item = new OhlcItem();
        item.setX(Instant.ofEpochMilli(md.get(0).getTimestamp()));
        item.setLow(md.get(0).getPrice());
        item.setHigh(md.get(3).getPrice());
        item.setClose(md.get(1).getPrice());
        item.setOpen(md.get(2).getPrice());
        if (dataSeries.size() > maxItemsInChart.get()) {
            dataSeries.add(item, true, true);
        } else {
            dataSeries.add(item, true, false);
        }
    }

    private void addDataSeriesPoint(DataSeries dataSeries, MarketData md) {
        DataSeriesItem item = new DataSeriesItem();
        item.setX(Instant.ofEpochMilli(md.getTimestamp()));
        item.setY(md.getPrice());
        if (dataSeries.size() > maxItemsInChart.get()) {
            dataSeries.add(item, true, true);
        } else {
            dataSeries.add(item, true, false);
        }
    }

    @Override
    public void afterNavigation(AfterNavigationEvent afterNavigationEvent) {
        getUI().ifPresent(ui -> new Thread(() -> {
            updateGrid(ui, rowsGrid1);
        }).start());
        getUI().ifPresent(ui -> new Thread(() -> {
            updateGrid(ui, rowsGrid2);
        }).start());
        getUI().ifPresent(ui -> new Thread(() -> {
            updateLineChart(ui, dataSeries1);
        }).start());
        getUI().ifPresent(ui -> new Thread(() -> {
            updateOHLCChart(ui, dataSeries2);
        }).start());
        getUI().ifPresent(ui -> ui.addDetachListener(event -> shouldStop.set(true)));
    }

    private void updateLineChart(UI ui, DataSeries dataSeries) {
        while (!shouldStop.get()) {
            MarketData randomMarketData = getRandomMarketData();
            ui.access(() -> {
                addDataSeriesPoint(dataSeries, randomMarketData);
                ui.push();
            });
            try {
                Thread.sleep((long) random.nextInt(sleepTimeoutMax.get()));
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    private void updateOHLCChart(UI ui, DataSeries dataSeries) {
        while (!shouldStop.get()) {
            ui.access(() -> {
                addDataSeriesPoints(dataSeries, getRandomMarketData(4));
                ui.push();
            });
            try {
                Thread.sleep((long) random.nextInt(sleepTimeoutMax.get()));
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    private void updateGrid(UI ui, LinkedList<Map<String, Span>> rows) {
        while (!shouldStop.get()) {
            List<MarketData> data = getRandomMarketData(numberofUpdatedItemsAtOnce.get());
            if (!rows.isEmpty()) {
                ui.access(() -> {
                    for (MarketData marketData : data) {
                        Map<String, Span> row = rows.get(random.nextInt(rows.size()));
                        row.get(COLUMN_TIMESTAMP).setText(formatDate(marketData.getTimestamp()));
                        row.get(COLUMN_BROKER).setText(marketData.getBroker().name());
                        row.get(COLUMN_EXCHANGE).setText(marketData.getExchange().name());
                        row.get(COLUMN_SYMBOL).setText(marketData.getSymbol());
                        row.get(COLUMN_CURRENCY).setText(marketData.getCurrency());
                        row.get(COLUMN_PRICE_TYPE).setText(marketData.getPriceType().name());
                        row.get(COLUMN_EVENT_TYPE).setText(marketData.getEventType().name());
                        row.get(COLUMN_PRICE).setText(marketData.getPrice().toString());
                        row.get(COLUMN_VOLUME).setText(marketData.getVolume().toString());
                    }
//                    ui.push();
                });
            }
            try {
                Thread.sleep((long) random.nextInt(sleepTimeoutMax.get()));
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void beforeLeave(BeforeLeaveEvent event) {
        shouldStop.set(true);
    }



    private String formatDate(long timestamp) {
        return DateTimeFormatter.ISO_TIME.format(LocalDateTime.ofInstant(Instant.ofEpochMilli(timestamp), ZoneId.systemDefault()));
    }

    private List<MarketData> getRandomMarketData(int count) {
        List<MarketData> returnList = new ArrayList<>();
        for (int x = 0; x < count;x++) {
            returnList.add(getRandomMarketData());
        }
        return returnList;
    }

    private MarketData getRandomMarketData() {
        return new MarketData(
                timestampBase.addAndGet(random.nextInt(1000)),
                EAsset.values()[random.nextInt(EAsset.values().length)].name(),
                EAsset.values()[random.nextInt(EAsset.values().length)].name(),
                BigDecimal.valueOf(random.nextDouble()),
                BigDecimal.valueOf(random.nextDouble()),
                EPriceType.values()[random.nextInt(EPriceType.values().length)],
                EEventType.values()[random.nextInt(EEventType.values().length)],
                EExchange.values()[random.nextInt(EExchange.values().length)],
                EBroker.values()[random.nextInt(EBroker.values().length)]
        );
    }
}






