package com.nordlicht.views.empty;

import java.math.BigDecimal;
import java.util.StringJoiner;

public class MarketData implements Comparable<MarketData> {

    private long timestamp;
    private String symbol;
    private String currency;
    private BigDecimal price;
    private BigDecimal volume;
    private EPriceType priceType;
    private EEventType eventType;
    private EExchange exchange;
    private EBroker broker;

    public MarketData() {
    }

    public MarketData(long timestamp, String symbol, String currency, BigDecimal price, BigDecimal volume,
                      EPriceType priceType, EEventType eventType, EExchange exchange, EBroker broker) {
        this.timestamp = timestamp;
        this.symbol = symbol;
        this.currency = currency;
        this.price = price;
        this.volume = volume;
        this.priceType = priceType;
        this.eventType = eventType;
        this.exchange = exchange;
        this.broker = broker;
    }

    @Override
    public int compareTo(MarketData o) {
        return price.compareTo(o.getPrice());
    }

    public long getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(long timestamp) {
        this.timestamp = timestamp;
    }

    public String getSymbol() {
        return symbol;
    }

    public void setSymbol(String symbol) {
        this.symbol = symbol;
    }

    public String getCurrency() {
        return currency;
    }

    public void setCurrency(String currency) {
        this.currency = currency;
    }

    public BigDecimal getPrice() {
        return price;
    }

    public void setPrice(BigDecimal price) {
        this.price = price;
    }

    public BigDecimal getVolume() {
        return volume;
    }

    public void setVolume(BigDecimal volume) {
        this.volume = volume;
    }

    public EPriceType getPriceType() {
        return priceType;
    }

    public void setPriceType(EPriceType priceType) {
        this.priceType = priceType;
    }

    public EEventType getEventType() {
        return eventType;
    }

    public void setEventType(EEventType eventType) {
        this.eventType = eventType;
    }

    public EExchange getExchange() {
        return exchange;
    }

    public void setExchange(EExchange exchange) {
        this.exchange = exchange;
    }

    public EBroker getBroker() {
        return broker;
    }

    public void setBroker(EBroker broker) {
        this.broker = broker;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof MarketData)) return false;

        MarketData that = (MarketData) o;

        if (timestamp != that.timestamp) return false;
        if (symbol != null ? !symbol.equals(that.symbol) : that.symbol != null) return false;
        if (currency != null ? !currency.equals(that.currency) : that.currency != null) return false;
        if (price != null ? !price.equals(that.price) : that.price != null) return false;
        if (volume != null ? !volume.equals(that.volume) : that.volume != null) return false;
        if (priceType != that.priceType) return false;
        if (eventType != that.eventType) return false;
        if (exchange != that.exchange) return false;
        return broker == that.broker;
    }

    @Override
    public int hashCode() {
        int result = (int) (timestamp ^ (timestamp >>> 32));
        result = 31 * result + (symbol != null ? symbol.hashCode() : 0);
        result = 31 * result + (currency != null ? currency.hashCode() : 0);
        result = 31 * result + (price != null ? price.hashCode() : 0);
        result = 31 * result + (volume != null ? volume.hashCode() : 0);
        result = 31 * result + (priceType != null ? priceType.hashCode() : 0);
        result = 31 * result + (eventType != null ? eventType.hashCode() : 0);
        result = 31 * result + (exchange != null ? exchange.hashCode() : 0);
        result = 31 * result + (broker != null ? broker.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return new StringJoiner(", ", MarketData.class.getSimpleName() + "[", "]")
                .add("timestamp=" + timestamp)
                .add("symbol='" + symbol + "'")
                .add("currency='" + currency + "'")
                .add("price=" + price)
                .add("volume=" + volume)
                .add("priceType=" + priceType)
                .add("eventType=" + eventType)
                .add("exchange=" + exchange)
                .add("broker=" + broker)
                .toString();
    }
}
